# -*- coding: utf-8 -*-
# This file is part of the lacie-nas project
#
# Copyright 2011 Benoît Canet
#
# Authors:
# - Benoît Canet <benoit.canet@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

from parser import *

def license():
	return """/*
 * This file is automaticaly generated by input-eventd's gen a script of the
 * lacie-nas project
 *
 * Code : git clone git://lacie-nas.org/nas-tools.git
 */
"""

def generate_header():
	result = """
#ifndef INPUT_TABLES_H
#define INPUT_TABLES_H

#include <stdint.h>
"""
	return result

def generate_footer():
	return "#endif\n"

def generate_struct():
	return """
struct input_entry {
	uint16_t value;
	char   	 string[128];
};	

"""

def generate_declaration(name):
	return "struct input_entry %s[] = {\n" % name

def generate_table(name, array):
	result = ""
	result += generate_declaration(name)

	for key in array:
		result += '\t{'
		result += '.value = %s, ' % key
		result += '.string = "%s"' % array[key]
		result += '},\n'
	
	result = result.strip()[:-1]
	result += "\n};\n\n"
	return result

def generate_index_struct():
	return """
struct index_entry {
	uint16_t type;
	uint16_t count;
	struct input_entry *codes;
};

"""

def generate_index(types, not_empty, codes_count):
	result = "struct index_entry table_index[] = {\n"

	for key in types:
		value = types[key][3:]
		if value in not_empty:
			value = "codes_"+ value.lower()
			result += "\t{.type = %s, .codes = %s, .count = %i},\n" % \
					 (key, value, codes_count[value])

	result = result.strip()[:-1]
	result += "\n};\n\n"
	return result

def generate(filename):
	result = license()
	result += generate_header()
	result += generate_struct()
	result += generate_index_struct()

	types = parse_file_by_prefix(filename, "EV_")
	result += generate_table("event_types", types)

	not_empty_types = []
	codes_count = {}
	for t in get_event_type_list(types):
		codes = parse_file_by_prefix(filename, t+"_")
		if len(codes):
			table_name = "codes_"+t.lower()
			result += generate_table(table_name, codes)
			not_empty_types.append(t)
			codes_count[table_name] = len(codes)

	result += generate_index(types, not_empty_types, codes_count)
	result += generate_footer()

	return result

